@@ -1,3 +1,25 @@
+/**
+ * 
+ * Hoel database abstraction library
+ * 
+ * h-private.h: private structures and functions declarations
+ * 
+ * Copyright 2018-2020 Nicolas Mora <mail@babelouest.org>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation;
+ * version 2.1 of the License.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU GENERAL PUBLIC LICENSE for more details.
+ *
+ * You should have received a copy of the GNU General Public
+ * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+ * 
+ */
 
 #ifndef __H_PRIVATE_H_
 #define __H_PRIVATE_H_
@@ -22,7 +44,7 @@ int h_result_add_row(struct _h_result * result, struct _h_data * row, int rows);
  * return pointer to the new structure
  * return NULL on error
  */
-struct _h_data * h_new_data_int(const int value);
+struct _h_data * h_new_data_int(const long long int value);
 
 /**
  * Allocate memory for a new struct _h_data * containing a double
