@@ -4,7 +4,7 @@
  * 
  * hoel-mariadb.c: Maria DB/Mysql specific functions
  * 
- * Copyright 2015-2016 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2015-2020 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -27,6 +27,7 @@
 /* MariaDB library Includes */
 #include <mysql.h>
 #include <string.h>
+#include <stdbool.h>
 
 /**
  * MariaDB handle
@@ -51,31 +52,36 @@ struct _h_mariadb {
 struct _h_connection * h_connect_mariadb(const char * host, const char * user, const char * passwd, const char * db, const unsigned int port, const char * unix_socket) {
   struct _h_connection * conn = NULL;
   pthread_mutexattr_t mutexattr;
-  my_bool reconnect = 1;
+  bool reconnect = 1;
+  o_malloc_t malloc_fn;
+  o_free_t free_fn;
+  
+  o_get_alloc_funcs(&malloc_fn, NULL, &free_fn);
+  json_set_alloc_funcs((json_malloc_t)malloc_fn, (json_free_t)free_fn);
 
   if (host != NULL && db != NULL) {
-    conn = malloc(sizeof(struct _h_connection));
+    conn = o_malloc(sizeof(struct _h_connection));
     if (conn == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for conn");
       return NULL;
     }
     
     conn->type = HOEL_DB_TYPE_MARIADB;
-    conn->connection = malloc(sizeof(struct _h_mariadb));
+    conn->connection = o_malloc(sizeof(struct _h_mariadb));
     if (conn->connection == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for conn->connection");
-      free(conn);
+      o_free(conn);
       return NULL;
     }
     if (mysql_library_init(0, NULL, NULL)) {
       y_log_message(Y_LOG_LEVEL_ERROR, "mysql_library_init error, aborting");
-      free(conn);
+      o_free(conn);
       return NULL;
     }
     ((struct _h_mariadb *)conn->connection)->db_handle = mysql_init(NULL);
     if (((struct _h_mariadb *)conn->connection)->db_handle == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "mysql_init error, aborting");
-      free(conn);
+      o_free(conn);
       return NULL;
     }
     if (mysql_real_connect(((struct _h_mariadb *)conn->connection)->db_handle,
@@ -83,7 +89,7 @@ struct _h_connection * h_connect_mariadb(const char * host, const char * user, c
       y_log_message(Y_LOG_LEVEL_ERROR, "Error connecting to mariadb database %s", db);
       y_log_message(Y_LOG_LEVEL_DEBUG, "Error message: \"%s\"", mysql_error(((struct _h_mariadb *)conn->connection)->db_handle));
       mysql_close(((struct _h_mariadb *)conn->connection)->db_handle);
-      free(conn);
+      o_free(conn);
       return NULL;
     } else {
       /* Set MYSQL_OPT_RECONNECT to true to reconnect automatically when connection is closed by the server (to avoid CR_SERVER_GONE_ERROR) */
@@ -115,7 +121,7 @@ void h_close_mariadb(struct _h_connection * conn) {
  * returned value must be free'd after use
  */
 char * h_escape_string_mariadb(const struct _h_connection * conn, const char * unsafe) {
-  char * escaped = malloc(2 * strlen(unsafe) + sizeof(char));
+  char * escaped = o_malloc(2 * strlen(unsafe) + sizeof(char));
   if (escaped == NULL) {
     y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for escaped");
     return NULL;
@@ -124,11 +130,26 @@ char * h_escape_string_mariadb(const struct _h_connection * conn, const char * u
   return escaped;
 }
 
+/**
+ * Escapes a string and returns it ready to be inserted in the query
+ * returned value must be free'd after use
+ */
+char * h_escape_string_with_quotes_mariadb(const struct _h_connection * conn, const char * unsafe) {
+  char * escaped = h_escape_string_mariadb(conn, unsafe), * escaped_returned = NULL;
+  if (escaped == NULL) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for escaped");
+    return NULL;
+  }
+  escaped_returned = msprintf("'%s'", escaped);
+  o_free(escaped);
+  return escaped_returned;
+}
+
 /**
  * Return the id of the last inserted value
  */
-int h_last_insert_id_mariadb(const struct _h_connection * conn) {
-  int id = mysql_insert_id(((struct _h_mariadb *)conn->connection)->db_handle);
+long long int h_last_insert_id_mariadb(const struct _h_connection * conn) {
+  long long int id = mysql_insert_id(((struct _h_mariadb *)conn->connection)->db_handle);
   if (id <= 0) {
     y_log_message(Y_LOG_LEVEL_ERROR, "Error executing mysql_insert_id");
     y_log_message(Y_LOG_LEVEL_DEBUG, "Error message: \"%s\"", mysql_error(((struct _h_mariadb *)conn->connection)->db_handle));
@@ -311,7 +332,7 @@ int h_execute_query_json_mariadb(const struct _h_connection * conn, const char *
  */
 struct _h_data * h_get_mariadb_value(const char * value, const unsigned long length, const int m_type) {
   struct _h_data * data = NULL;
-  int i_value;
+  long long int i_value;
   double d_value;
   struct tm tm_value;
   char * endptr;
@@ -326,7 +347,7 @@ struct _h_data * h_get_mariadb_value(const char * value, const unsigned long len
       case FIELD_TYPE_LONGLONG:
       case FIELD_TYPE_INT24:
       case FIELD_TYPE_YEAR:
-        i_value = strtol(value, &endptr, 10);
+        i_value = strtoll(value, &endptr, 10);
         if (endptr != value) {
           data = h_new_data_int(i_value);
         } else {
