@@ -4,7 +4,7 @@
  * 
  * hoel-simple-json.c: hoel simple Json query functions
  * 
- * Copyright 2015-2016 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2015-2020 Nicolas Mora <mail@babelouest.org>
  *
  * This program is o_free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -36,13 +36,13 @@ static char * h_get_insert_values_from_json_object(const struct _h_connection *
   json_object_foreach((json_t *)data, key, value) {
     switch (json_typeof(value)) {
       case JSON_STRING:
-        escape = h_escape_string(conn, json_string_value(value));
+        escape = h_escape_string_with_quotes(conn, json_string_value(value));
         if (escape == NULL) {
           y_log_message(Y_LOG_LEVEL_ERROR, "Hoel/h_get_insert_values_from_json_object - Error escape");
           o_free(new_data);
           new_data = NULL;
         } else {
-          new_data = msprintf("'%s'", escape);
+          new_data = msprintf("%s", escape);
           o_free(escape);
         }
         break;
@@ -116,14 +116,14 @@ static char * h_get_insert_columns_from_json_object(json_t * data) {
 
   json_object_foreach((json_t *)data, key, value) {
     if (i == 0) {
-      insert_cols = msprintf("`%s`", key);
+      insert_cols = msprintf("%s", key);
       if (insert_cols == NULL) {
         y_log_message(Y_LOG_LEVEL_ERROR, "Hoel/h_get_insert_columns_from_json_object - Error allocating insert_cols");
         return NULL;
       }
       i = 1;
     } else {
-      tmp = msprintf("%s,`%s`", insert_cols, key);
+      tmp = msprintf("%s,%s", insert_cols, key);
       o_free(insert_cols);
       if (tmp == NULL) {
         y_log_message(Y_LOG_LEVEL_ERROR, "Hoel/h_get_insert_columns_from_json_object - Error allocating insert_cols");
@@ -148,7 +148,7 @@ static char * h_get_insert_query_from_json_object(const struct _h_connection * c
   if (insert_cols == NULL || insert_data == NULL) {
     y_log_message(Y_LOG_LEVEL_ERROR, "Hoel/h_get_insert_query_from_json_object - Error h_get_insert_columns_from_json_object or h_get_insert_values_from_json_object");
   } else {
-    to_return = msprintf("INSERT INTO `%s` (%s) VALUES %s", table, insert_cols, insert_data);
+    to_return = msprintf("INSERT INTO %s (%s) VALUES %s", table, insert_cols, insert_data);
   }
   o_free(insert_cols);
   o_free(insert_data);
@@ -171,7 +171,7 @@ static char * h_get_insert_query_from_json_array(const struct _h_connection * co
     insert_data = h_get_insert_values_from_json_object(conn, j_row);
     if (!index) {
       insert_cols = h_get_insert_columns_from_json_object(j_row);
-      to_return = msprintf("INSERT INTO `%s` (%s) VALUES %s", table, insert_cols, insert_data);
+      to_return = msprintf("INSERT INTO %s (%s) VALUES %s", table, insert_cols, insert_data);
       o_free(insert_cols);
       if (to_return == NULL) {
         y_log_message(Y_LOG_LEVEL_ERROR, "Hoel/h_get_insert_query_from_json_array - Error allocating to_return");
@@ -255,14 +255,14 @@ static char * h_get_where_clause_from_json_object(const struct _h_connection * c
                     return NULL;
                   } else {
                     if (json_is_string(j_element)) {
-                      escape = h_escape_string(conn, json_string_value(j_element));
+                      escape = h_escape_string_with_quotes(conn, json_string_value(j_element));
                       if (escape == NULL) {
                         y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error escape");
                         o_free(clause);
                         o_free(where_clause);
                         return NULL;
                       }
-                      dump = msprintf("'%s'", escape);
+                      dump = msprintf("%s", escape);
                       o_free(escape);
                     } else if (json_is_real(j_element)) {
                       dump = msprintf("%f", json_real_value(j_element));
@@ -293,13 +293,13 @@ static char * h_get_where_clause_from_json_object(const struct _h_connection * c
               } else if (json_is_integer(val)) {
                 clause = msprintf("%s %s %" JSON_INTEGER_FORMAT, key, json_string_value(ope), json_integer_value(val));
               } else {
-                escape = h_escape_string(conn, json_string_value(val));
+                escape = h_escape_string_with_quotes(conn, json_string_value(val));
                 if (escape == NULL) {
                   y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error escape");
                   o_free(where_clause);
                   return NULL;
                 }
-                clause = msprintf("%s %s '%s'", key, json_string_value(ope), escape);
+                clause = msprintf("%s %s %s", key, json_string_value(ope), escape);
                 o_free(escape);
               }
             }
@@ -313,13 +313,13 @@ static char * h_get_where_clause_from_json_object(const struct _h_connection * c
           if (json_is_null(value)) {
             clause = msprintf("%s IS NULL", key);
           } else if (json_is_string(value)) {
-            escape = h_escape_string(conn, json_string_value(value));
+            escape = h_escape_string_with_quotes(conn, json_string_value(value));
             if (escape == NULL) {
               y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error escape");
               o_free(where_clause);
               return NULL;
             }
-            clause = msprintf("%s='%s'", key, escape);
+            clause = msprintf("%s=%s", key, escape);
             o_free(escape);
           } else if (json_is_integer(value)) {
             clause = msprintf("%s='%"JSON_INTEGER_FORMAT"'", key, json_integer_value(value));
@@ -395,8 +395,8 @@ static char * h_get_set_clause_from_json_object(const struct _h_connection * con
         return NULL;
       } else {
         if (json_is_string(value)) {
-          tmp = h_escape_string(conn, json_string_value(value));
-          escape = msprintf("'%s'", tmp);
+          tmp = h_escape_string_with_quotes(conn, json_string_value(value));
+          escape = msprintf("%s", tmp);
           o_free(tmp);
         } else if (json_is_real(value)) {
           escape = msprintf("%f", json_real_value(value));
@@ -418,9 +418,9 @@ static char * h_get_set_clause_from_json_object(const struct _h_connection * con
         }
         if (i == 0) {
           if (!json_is_null(value)) {
-            where_clause = msprintf("`%s`=%s", key, escape);
+            where_clause = msprintf("%s=%s", key, escape);
           } else {
-            where_clause = msprintf("`%s`=NULL", key);
+            where_clause = msprintf("%s=NULL", key);
           }
           if (where_clause == NULL) {
             y_log_message(Y_LOG_LEVEL_DEBUG, "Hoel/h_get_set_clause_from_json_object - Error where_clause");
@@ -429,9 +429,9 @@ static char * h_get_set_clause_from_json_object(const struct _h_connection * con
           i = 1;
         } else {
           if (!json_is_null(value)) {
-            tmp = msprintf("%s, `%s`=%s", where_clause, key, escape);
+            tmp = msprintf("%s, %s=%s", where_clause, key, escape);
           } else {
-            tmp = msprintf("%s, `%s`=NULL", where_clause, key);
+            tmp = msprintf("%s, %s=NULL", where_clause, key);
           }
           o_free(where_clause);
           if (tmp == NULL) {
@@ -537,15 +537,16 @@ int h_select(const struct _h_connection * conn, const json_t * j_query, json_t *
     } else {
       str_where_limit = msprintf("LIMIT %" JSON_INTEGER_FORMAT, limit);
     }
-    if (str_where_limit == NULL) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for str_where_limit");
-      o_free(columns);
-      o_free(where_clause);
-      return H_ERROR_MEMORY;
-    }
   } else {
     str_where_limit = o_strdup("");
   }
+  if (str_where_limit == NULL) {
+    y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for str_where_limit");
+    o_free(columns);
+    o_free(where_clause);
+    return H_ERROR_MEMORY;
+  }
+
   if (order_by != NULL && json_is_string(order_by)) {
     str_order_by = msprintf("ORDER BY %s", json_string_value(order_by));
   } else {
@@ -559,16 +560,7 @@ int h_select(const struct _h_connection * conn, const json_t * j_query, json_t *
     return H_ERROR_MEMORY;
   }
   
-  if (str_order_by == NULL) {
-    y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for str_order_by");
-    o_free(columns);
-    o_free(where_clause);
-    o_free(str_where_limit);
-    o_free(str_order_by);
-    return H_ERROR_MEMORY;
-  }
-  
-  query = msprintf("SELECT %s FROM `%s` WHERE %s %s %s", columns, table, where_clause, str_order_by, str_where_limit);
+  query = msprintf("SELECT %s FROM %s WHERE %s %s %s", columns, table, where_clause, str_order_by, str_where_limit);
   if (query == NULL) {
     y_log_message(Y_LOG_LEVEL_DEBUG, "Hoel/h_select Error allocating query");
     o_free(columns);
@@ -664,7 +656,7 @@ int h_insert(const struct _h_connection * conn, const json_t * j_query, char **
 /**
  * h_last_insert_id
  * return the id of the last inserted value
- * return a pointer to `json_t *` on success, NULL otherwise.
+ * return a pointer to json_t * on success, NULL otherwise.
  * The returned value is of type JSON_INTEGER
  */
 json_t * h_last_insert_id(const struct _h_connection * conn) {
@@ -729,10 +721,10 @@ int h_update(const struct _h_connection * conn, const json_t * j_query, char **
   if (json_is_object(json_object_get(j_query, "where")) && json_object_size(json_object_get(j_query, "where")) > 0) {
     where = json_object_get(j_query, "where");
     where_clause = h_get_where_clause_from_json_object(conn, where);
-    query = msprintf("UPDATE `%s` SET %s WHERE %s", table, set_clause, where_clause);
+    query = msprintf("UPDATE %s SET %s WHERE %s", table, set_clause, where_clause);
     o_free(where_clause);
   } else {
-    query = msprintf("UPDATE `%s` SET %s", table, set_clause);
+    query = msprintf("UPDATE %s SET %s", table, set_clause);
   }
   
   o_free(set_clause);
@@ -776,10 +768,10 @@ int h_delete(const struct _h_connection * conn, const json_t * j_query, char **
       y_log_message(Y_LOG_LEVEL_DEBUG, "Hoel/h_delete - Error invalid input parameters");
       return H_ERROR_PARAMS;
     }
-    query = msprintf("DELETE FROM `%s` WHERE %s", table, where_clause);
+    query = msprintf("DELETE FROM %s WHERE %s", table, where_clause);
     o_free(where_clause);
   } else {
-    query = msprintf("DELETE FROM `%s`", table);
+    query = msprintf("DELETE FROM %s", table);
   }
   
   if (query == NULL) {
